(*
   AnotherReg
   Registry-aware Components Collection
   (C) Glen Why, 1996
   Version 01.04.00

Version History

   01.00.00 - first
   01.00.01 - Small bug in TVarinatProfile.SetValue fixed
   01.00.02 - TCustomKey.SetParentKey moved to the protected
              stub and made vitrual;

   01.01.00 - + TSoundEvent 
   01.01.01 - Automatic search for the AppConfigKey added
              to the TSoundEvent.Create;
   01.01.02 - TSoundEvent.EventLabel property added
   01.01.03 - TSoundEvent.OnSoundEvent event added
   01.01.04 - 01.01.03's bugs fixed

   01.02.00 - + TStringProfile

   01.03.00 - + TComponentProfile

   01.04.00 - + TFormRestorer

notes:

* According to Win32 Developer's reference, NT's version
  of RegDeleteKey will fail if specified key has subkeys,
  as a result ERegistryError will be raised on call to
  the Delete method of key that has subkey(s). Nothing
  to worry about in Windows 95.

*)
unit AnotherReg;

interface
uses
 Windows, MMSystem, Classes, ExtCtrls, SysUtils, Forms;


const

  SndEvent_DefAutoRegister = false;
  restorer_ValueName = 'SizeAndPosition';


type

  ERegistryError = class( Exception );

  TRegistryEntry = class( TComponent )
  protected
    function GetExist :Boolean; virtual; abstract;
    procedure DoDelete; virtual; abstract;
  public
    procedure Delete;
    property Exist :Boolean read GetExist;
  end;

  TCustomKey = class( TRegistryEntry )
  private
    FParentKey :TCustomKey;
    FAutoCloseTimer :TTimer;
    FHandle :THandle;
    procedure SetActive( V :Boolean );
    function GetActive :Boolean;
    function GetHandle :THandle;
    procedure SetAutoCloseTimer;
    procedure KillAutoCloseTimer;
    procedure AutoClose( sender :TObject );
    function GetFullPath :string;
  protected
    procedure SetParentKey( V :TCustomKey ); virtual;
    function GetExist :Boolean; override;
    procedure Notification( AComponent :TComponent;
     Operation :TOperation ); override;
    procedure DoDelete; override;
    function GetPath :string; virtual;
    procedure Open; virtual;
    procedure Close; virtual;
    property ParentKey :TCustomKey read FParentKey write SetParentKey;
  public
    constructor Create( anOwner :TComponent ); override;
    destructor Destroy; override;
    property Active :Boolean read GetActive write SetActive stored false;
    property Handle :THandle read GetHandle;
    property Path :String read GetPath;
    property FullPath :String read GetFullPath;
  end;

  TRegKey = class( TCustomKey )
  published
    property ParentKey;
  end;

  TAppConfigKey = class( TCustomKey )
  private
    FCompanyName :String;
    FApplicationName :String;
    FApplicationVersion :String;
    procedure SetApplicationName( V :String );
    procedure SetCompanyName( V :String );
    procedure SetApplicationVersion( V :String );
  protected
    constructor Create( anOwner :TComponent ); override;
    function GetPath :string; override;
  published
    property ApplicationName :String
      read FApplicationName write SetApplicationName;
    property ApplicationVersion :String
      read FApplicationVersion write SetApplicationVersion;
    property CompanyName :string
      read FCompanyName write SetCompanyName;
  end;

  TCustomProfile = class( TRegistryEntry )
  private
    FKey :TCustomKey;
  protected
    function GetExist :boolean; override;
    procedure Notification( AComponent :TComponent;
     Operation :TOperation ); override;
    procedure DoDelete; override;
  public
    constructor Create( anOwner :TComponent ); override;
    property Key :TCustomKey
      read FKey write FKey;
  end;

  TVariantProfile = class( TCustomProfile )
  protected
    function GetValue :Variant; virtual;
    procedure SetValue( V :Variant ); virtual;
  public
    property Value :Variant read GetValue write SetValue;
  published
    property Key;
  end;


  TSoundEventOption = ( seResetSchemeOnUpdate, sePlaySync,
    seNoDefault, seNoStop );

  TSoundEventOptions = set of TSoundEventOption;

  TSoundEvent = class( TCustomKey )
  private
    FEventLabel :String;
    FAutoRegister :Boolean;
    FDefaultSoundName :String;
    FAppConfigKey :TAppConfigKey;
    FOptions :TSoundEventOptions;
    FOnSoundEvent :TNotifyEvent;
    function GetSOundName :String;
    procedure SetSoundName( const NewName :String );
    function GetDefaultSoundName :String;
    procedure SetDefaultSoundName( const NewName :String );
    function ReadSoundName :string;
    procedure WriteSoundName( const NewName :String );
    function ReadDefaultSoundName :string;
    procedure WriteDefaultSoundName( const NewName :String );
    procedure SetEventLabel( const NewLabel :String );
  protected
    function GetPath :string; override;
    procedure Loaded; override;
    procedure SoundEvent; dynamic;
    procedure DoDelete; override;
  public
    constructor Create( anOwner :TComponent ); override;
    destructor Destroy; override;
    procedure RegisterSoundEvent;
    procedure UnregisterSoundEvent;
    procedure Play; 
    property SoundName :String
      read GetSoundName write SetSoundName stored false;
  published
    property AppConfigKey :TAppCOnfigKey
      read FAppConfigKey write FAppConfigKey;
    property AutoRegister :boolean
      read FAutoRegister write FAutoRegister default SndEvent_DefAutoRegister;
    property Options :TSoundEventOptions
      read FOptions write FOptions;
    property DefaultSoundName :String
      read GetDefaultSoundName write SetDefaultSoundName;
    property EventLabel :String
      read FEventLabel write SetEventLabel;
    property OnSoundEvent :TNotifyEvent
      read FOnSoundEvent write FOnSoundEvent;
  end;

  TStringProfile = class( TCustomProfile )
  private
    procedure SetValue( NewValue :String );
    function GetValue :String;
  public
    property Value :string
      read GetValue write SetValue;
  published
    property Key;
  end;

  TComponentProfile = class( TCustomProfile )
  private
    procedure SetValue( NewValue :TComponent );
    function GetValue :TComponent;
  public
    property Value :TComponent
      read GetValue write SetValue;
  published
    property Key :TCustomKey
      read FKey write FKey;
  end;


  TFormRestorerOption = ( frSizeAndPosition, frSizeOnly, frPositionOnly );

  TFormRestorer = class( TCustomKey )
  private
    FRestoring :TFormRestorerOption;
    FOldOnClose :TCloseEvent;
    procedure OurOnClose( Sender :TObject; var Action :TCloseAction );
  protected
    function GetPath :string; override;
    procedure Loaded; override;
  public
    constructor Create( anOwner :TComponent ); override;
    procedure Store;
    procedure Restore;
  published
    property Restoring :TFormRestorerOption
      read FRestoring write FRestoring default frSizeAndPosition;
    property ParentKey;
  end;

implementation

{$R ANOTHERREG.RES}

{$I SYSUTILS.INC}
{$I ANOTHERREG.INC}


const

AutoCloseInterval = 5000;

function FindOwnerForm( C :TComponent ):TForm;
begin
 result := nil;
 if c = nil then exit;
 result := TForm( c.Owner );
 while result <> nil do
  if result is TForm
    then break
    else result := TForm( result.Owner );
end;  

function StrToIdent( const S :String ):String;
var i :integer;
begin
 if IsValidIdent( S )
  then result := S
  else begin
    if ( S[ 1 ] in [ '0'..'9' ] )
     then result := '_' + Trim( S )
     else result := Trim( S );
    for i := 2 to Length( result ) - 1 do
     if result[ i ] = ' ' then result[ i ] := '_';
  end;
end;


function FindCompByClass( Root :TComponent; CompClass :TClass ):TComponent;
var i :integer;
begin
  result := nil;
  if ( root = nil ) then exit;
  for i := 0 to root.componentCount - 1 do
   begin
     if root.components[i] is CompClass
       then result := root.components[i]
       else result := FindCompByClass( root.components[i], CompClass );
     if result <> nil then break;
   end;
end;


function CheckCrossRef( Root, Item :TCustomKey ):boolean;
var K :TCustomKey;
begin
  result := false;
  if ( Item = nil ) or ( root = nil ) then exit;
  K := Item;
  while K <> Nil do
   begin
    if K = Root then
     begin
      result := true;
      exit;
     end
    else K := K.FParentKey;
   end;
end;


{ TCustomLey }

procedure TCustomKey.DoDelete;
var H :HKey;
begin
  close;
  if FParentKey <> Nil
    then H := FParentKey.Handle
    else H := HKEY_CURRENT_USER;
  if RegDeleteKey( H, PChar( Path ) ) <> ERROR_SUCCESS
   then raise ERegistryError.CreateResFmt( SDeleteKeyError, [ FullPath ] );
end;

function TCustomKey.GetExist :Boolean;
var H :THandle;
begin
 result := RegOpenKey( HKEY_CURRENT_USER, PChar( FullPath ), H ) = ERROR_SUCCESS;
 if result then RegCloseKey( H );
end;


procedure TCustomKey.Notification( AComponent :TComponent;
 Operation :TOperation );
begin
 if ( Operation = opRemove ) and ( AComponent = FParentKey )
  then FParentKey := nil;
end;


procedure TCustomKey.AutoClose( sender :TObject );
begin
 close;
end;


procedure TCustomKey.SetParentKey( V :TCustomKey );
begin
  if V = FParentKey then exit;
  if ( csLoading in ComponentSTate ) then
    begin
     FParentKey := V;
     exit;
    end;
  if CheckCrossRef( Self, V )
   then raise ERegistryError.CreateRes( SCrossRef );
  close;
  FParentKey := V;
end;

procedure TCustomKey.SetActive( V :Boolean );
begin
  if v then Open else Close;
end;

function TCustomKey.GetActive :Boolean;
begin
  result := FHandle <> 0;
end;

procedure TCustomKey.Open;
var H :THandle;
begin
 if ( FHandle = 0 ) then
  begin
   if ( FParentKey <> nil )
     then H := FParentKey.Handle
     else H := HKEY_CURRENT_USER;
   if RegCreateKey( H, PChar( Path ), FHandle ) <> ERROR_SUCCESS
     then raise ERegistryError.CreateResFmt( SOpenKeyError, [ Path ] );
  end;
 if ( FParentKey = nil ) then SetAutoCloseTimer;
end;

procedure TCustomKey.Close;
begin
 if FHandle = 0 then exit;
 if RegCloseKey( FHandle ) <> ERROR_SUCCESS
  then raise ERegistryError.CreateResFmt( SCloseKeyError, [ Path ] );
 FHandle := 0;
 if ( FParentKey = Nil ) then KillAutoCloseTimer;
end;

constructor TCustomKey.Create( anOwner :TComponent );
begin
 inherited Create( anOwner );
 FAutoCloseTimer := Nil;
 FHandle := 0;
 FParentKey := Nil;
end;

destructor TCustomKey.Destroy;
begin
 Close;
 inherited Destroy;
end;

function TCustomKey.GetHandle;
begin
 open;
 result := FHandle;
end;


procedure TCustomKey.SetAutoCloseTimer;
begin
 if FAutoCloseTimer = nil then FAutoCloseTimer := TTimer.Create( Self );
 with FAutoCloseTimer do
  begin
   OnTimer := AutoClose;
   Interval := AutoCloseInterval;
  end;
end;

procedure TCustomKey.KillAutoCloseTimer;
begin
 if FAutoCloseTimer <> nil then
  begin
   FAutoCloseTimer.Enabled := false;
   FAutoCloseTimer.Free;
   FAutoCloseTimer := nil;
  end;
end;

function TCustomKey.GetPath :string;
begin
 if ( FParentKey = Nil )
  then result := LoadStr( SLostKeyPath ) + '\' + Name
  else result := Name;
end;

function TCustomKey.GetFullPath :string;
begin
 if ( FParentKey <> Nil )
  then Result := FParentKey.FullPath + '\' + Path
  else result := path;
end;


{ TAppConfigKey }



procedure TAppConfigKey.SetApplicationName( V :String );
begin
  if V = '' then raise ERegistryError.CreateRes( SInvalidValue );
  close;
  FApplicationName := V;
end;

procedure TAppConfigKey.SetApplicationVersion( V :String );
begin
  if V = '' then raise ERegistryError.CreateRes( SInvalidValue );
  close;
  FApplicationVersion := V;
end;

constructor TAppConfigKey.Create( anOwner :TComponent );
begin
 inherited Create( anOwner );
 FCompanyName := LoadStr( SDefCompanyName );
 FApplicationName := LoadStr( SDefApplicationName );
 FApplicationVersion := LoadStr( SDefApplicationVersion );
end;

function TAppConfigKey.GetPath :string;
begin
 result := 'Software\' + CompanyName + '\' +
  ApplicationName + '\' + ApplicationVersion;
end;

procedure TAppConfigKey.SetCompanyName( V :String );
begin
  if V = '' then raise ERegistryError.CreateRes( SInvalidValue );
  close;
  FCompanyName := V;
end;

{ TCustomProfile }

procedure TCustomProfile.Notification( AComponent :TComponent;
 Operation :TOperation );
begin
  if ( AComponent = FKey ) and ( Operation = opRemove ) then FKey := Nil;
  inherited Notification( AComponent, Operation );
end;

constructor TCustomProfile.Create( anOwner :TComponent );
begin
 inherited Create( anOwner );
 FKey := Nil;
end;

function TCustomProfile.GetExist :boolean;
var T, S :Longint;
begin
  if Key = nil then raise ERegistryError.CreateRes( SNoKey );
  T := REG_BINARY;
  result := RegQueryValueEx( Key.Handle, PChar( Name ),
   nil, @T, nil, @S ) = ERROR_SUCCESS;
end;

procedure TCustomProfile.DoDelete;
begin
  if Key = nil then raise ERegistryError.CreateRes( SNoKey );
  if RegDeleteValue( Key.Handle, PChar( Name ) ) <> ERROR_SUCCESS then
   raise ERegistryError.CreateResFmt( SDeleteValueError, [ Name, Key.Path ] );
end;


{ TVariantProfile }

{ The code looks bizzare, but I had no time and information
 to do something more professional.
 By the way, can anybody make me a present
 called "recent version of Turbo Debugger?" }

function TVariantProfile.GetValue :Variant;
var S, R, T :Longint; B :Pointer;
begin
  if Key = nil then raise ERegistryError.CreateRes( SNoKey );
  T := REG_BINARY;
  R := RegQueryValueEx( Key.Handle, PChar( Name ), nil, @T, nil, @S );
  if ( R <> ERROR_SUCCESS ) then
    begin
     VarClear( result );
     exit;
    end;
  GetMem( B, S + 32 );
  if ( B = Nil ) then raise EOutOfMemory.CreateRes( SOutOfMemory );
  try
    R := RegQueryValueEx( Key.Handle, PChar( Name ), nil, @T, B, @S );
    if ( R <> ERROR_SUCCESS ) then raise ERegistryError.CreateRes( SRegReadError );
    if ( PVarData( B )^.VType = varString )
     then result := String( PChar( Longint( B ) + 2 ) )
     else move( b^, result, sizeof( variant ) );
  finally
    FreeMem( B );
  end;
end;


procedure TVariantProfile.SetValue( V :Variant );
var B :Pointer; R, S :Longint;
begin
  if Key = nil then raise ERegistryError.CreateRes( SNoKey );
  case VarType( V ) of
    varString : S := 2 + Length( V ) + 1;
    varByte, varSmallint, varInteger,
    varSingle, varDouble, varCurrency,
    varBoolean, varDate : S := SizeOf( Variant );
    else raise ERegistryError.CreateRes( SInvalidType );
  end;
  GetMem( B, S + 32 );
  if ( B = Nil ) then raise EOutOfMemory.CreateRes( SOutOfMemory );
  try
    move( V, B^, SizeOf( Variant ) );
    if ( ( VarType( V ) = varString ) and ( Length( V ) > 0 ) ) then
     move( TVarData( V ).VString^,  Pointer( Longint( B ) + 2 )^, S - 2  );
    R := RegSetValueEx( Key.Handle, PChar( Name ), 0, REG_BINARY, B, S );
    if ( R <> ERROR_SUCCESS ) then raise ERegistryError.CreateRes( SRegWriteError );
  finally
    FreeMem( B );
  end;
end;


{ TRegistryEntry }

procedure TRegistryEntry.Delete;
begin
 if exist then DoDelete;
end;


{ Predefined keys used by TSoundEvent }

type


  TSchemesKey = class( TCustomKey )
  protected
    function GetPath :string; override;
  end;

  TEventLabelsKey = class( TCustomKey )
  protected
    function GetPath :string; override;
  end;

  TSchemesAppsKey = class( TCustomKey )
  protected
    function GetPath :string; override;
  end;

  TAppEventsKey = class( TCustomKey )
  protected
    function GetPath :string; override;
  end;


function TAppEventsKey.GetPath :string;
begin
  result := 'AppEvents';
end;

function TSchemesKey.GetPath :string;
begin
 result := 'AppEvents\Schemes';
end;

function TSchemesAppsKey.GetPath :string;
begin
 result := 'AppEvents\Schemes\Apps';
end;

function TEventLabelsKey.GetPath :string;
begin
 result := 'AppEvents\EventLabels';
end;



{ TSoundEvent }



constructor TSoundEvent.Create( anOwner :TComponent );
var i :integer;
begin
 inherited Create( anOwner );
 FAutoRegister := SndEvent_DefAutoRegister;
 FDefaultSoundName := '';
 FOptions := [ seResetSchemeOnUpdate ];
 FAppConfigKey := Nil;
 FEventLabel := LoadStr( SDefSoundEventLabel );
 if ( csDesigning in ComponentState ) then
  for i := 0 to Screen.FormCount - 1 do
   begin
    FAppConfigKey := TAppConfigKey( FindCompByClass(
     Screen.Forms[ i ], TAppConfigKey ) );
    if ( FAppConfigKey <> Nil ) then break;
   end;
 FParentKey := TSchemesAppsKey.Create( Self );
end;


function TSoundEvent.GetSoundName :String;
begin
  result := ReadSoundName;
end;

procedure TSoundEvent.SetSoundName( const NewName :String );
begin
  WriteSoundName( NewName );
end;

{ DefaultSoundName property read accessor. At design-time
returns value of FDefaultSoundName. At run-time returns
default value of ".Default" subkey }

function TSoundEvent.GetDefaultSoundName :String;
begin
  if ( csDesigning in ComponentState ) or ( csLoading in ComponentState )
   then Result := FDefaultSoundName
   else result := ReadDefaultSoundName;
end;

{ DefaultSoundName property write accessor. At design-time stores
 new name in FDefaultSoundName field. At run-time updates
 default value of ".Default" subkey.)}

procedure TSoundEvent.SetDefaultSoundName( const NewName :String );
begin
  if ( csDesigning in ComponentState ) or ( csLoading in ComponentState )
   then FDefaultSoundName := NewName
   else WriteDefaultSoundName( NewName );
end;

{ Changes parent key. Accepts only TAppConfigKey }

function TSoundEvent.GetPath :string;
begin
  if ( FParentKey = nil )
   then result := 'LostEvents'
    else result := StrToIdent( FAppCOnfigKey.ApplicationName );
  result := result + '\' + Name;
end;

{ At run-time registers event if AutoRegister is true }

procedure TSoundEvent.Loaded;
begin
 inherited Loaded;
 if ( not ( csDesigning in ComponentState ) ) and AutoRegister
  then RegisterSoundEvent;
end;

function RegCreateSubKey( Key : hKey; const SubKeyName :string ):HKey;
var R, D :Longint; H :Thandle;
begin
  R := RegCreateKeyEx( Key, PChar( SubKeyName ), 0, nil, 0, KEY_ALL_ACCESS, nil, H, @D );
  if ( R <> ERROR_SUCCESS ) then
   raise ERegistryError.CreateResFmt( SOpenKeyError, [ SubKeyName ] );
  result := H;
end;

function RegReadSubKeyString(
 Key :hKey;
 const SubKeyName :string;
 const ValueName :string ):string;
var S, R, T :Longint; B :Pointer; H :THandle;
begin
  H := RegCreateSubKey( Key, SubKeyName );
  try
    T := REG_SZ;
    R := RegQueryValueEx( H, PChar( ValueName ), nil, @T, nil, @S );
    if ( R <> ERROR_SUCCESS ) or ( S = 0 )
     then result := ''
     else begin
       GetMem( B, S + 32 );
       if ( B = nil ) then raise EoutOfMemory.CReateRes( SOutOfMemory );
       try
         R := RegQueryValueEx( H, PChar( ValueName ), nil, @T, B, @S );
         if ( R <> ERROR_SUCCESS ) then raise ERegistryError.CreateRes( SRegReadError );
         result := PChar( B );
       finally
         FreeMem( B );
       end;
     end;
  finally
    RegCloseKey( H );
  end;
end;

procedure RegWriteSubKeyString(
 Key :hKey;
 const SubKeyName :string;
 const ValueName :string;
 const Value :String );
var R :Longint; H :THandle;
begin
  H := RegCreateSubKey( Key, SubKeyName );
  try
    R := RegSetValueEx( H, PChar( ValueName ), 0, REG_SZ,
     PChar( Value ), Length( Value ) + 1 );
    if ( R <> ERROR_SUCCESS ) then raise ERegistryError.CreateRes( SRegWriteError );
  finally
    RegCloseKey( H );
  end;
end;

const

path_Default = '.Default';
path_Current = '.current';
path_Schemes = 'Schemes';


function TSoundEvent.ReadDefaultSoundName :string;
begin
  result := RegReadSubKeyString( Handle, path_Default, '' );
end;

procedure TSoundEvent.WriteDefaultSoundName( const NewName :String );
begin
 RegWriteSubKeyString( Handle, path_Default, '', NewName );
end;

function TSoundEvent.ReadSoundName :string;
begin
  result := RegReadSubKeyString( Handle, path_Current, '' );
end;

procedure TSoundEvent.WriteSoundName( const NewName :String );
var SchemeName :String;
begin
 with TAppEventsKey.Create( Self ) do
  try
    if ( seResetSchemeOnUpdate in Options )
      then RegWriteSubKeyString( Handle, path_Schemes, '', path_Current )
      else begin
        SchemeName := RegReadSubKeyString( Handle, path_Schemes, '' );
        RegWriteSubKeyString( Self.Handle, SchemeName, '', NewName );
      end;
  finally
   free;
  end;
 RegWriteSubKeyString( Handle, path_Current, '', NewName );
end;

procedure TSoundEvent.RegisterSoundEvent;
begin
  if AppConfigKey = nil
   then raise ERegistryError.CreateRes( SLostSoundEventReg );
  { Setting \AppEvents\Schemes\Apps\<AppID> key }
  with TSchemesAppsKey.Create( Self ) do
   try
    RegWriteSubKeyString( Handle, StrToIdent( AppConfigKey.ApplicationName ),
     '', AppConfigKey.ApplicationName );
   finally
    Free;
   end;
  { Setting \AppEvents\EventLabels\<Name> key }
  with TEventLabelsKey.Create( Self ) do
   try
     RegWriteSubKeyString( Handle, Self.Name, '', EventLabel );
   finally
     free;
   end;
  { Setting up \AppEvents\Schemes\Apps\<AppID>\<Name>\.Default key }
  if FDefaultSoundName <> ''
   then RegWriteSubKeyString( Handle, path_Default, '', FDefaultSoundName );
  RegCreateSubKey( Handle, path_Current );
end;

procedure TSoundEvent.UnregisterSoundEvent;
begin
  Delete;
end;

procedure TSoundEvent.SetEventLabel( const NewLabel :String );
begin
 if EventLabel = NewLabel then exit;
 if ( NewLabel = '' ) then raise ERegistryError.CreateRes( SSoundEventLabel );
 FEventLabel := NewLabel;
end;

procedure TSoundEvent.SoundEvent;
begin
 if Assigned( FOnSoundEvent ) then FOnSOundEvent( Self );
end;

procedure TSoundEvent.Play;
var F :Longint; S :String;
begin
  SoundEvent;
  F := 0;
  if ( sePlaySync in Options ) then F := F OR SND_SYNC else F := F OR SND_ASYNC;
  if ( seNoDefault in Options ) then F := F OR SND_NODEFAULT;
  if ( seNoStop in Options ) then F := F OR SND_NOSTOP;
  S := SoundName;
  if ( S = '' ) and ( not ( seNoDefault in Options ) ) then S := DefaultSoundName;
  PlaySound( PChar( S ), 0, SND_FILENAME OR F );
end;

destructor TSoundEvent.Destroy;
begin
 PlaySound( nil, 0 , SND_PURGE );
 inherited Destroy;
end;

procedure TSoundEvent.DoDelete;
begin
 inherited DoDelete;
 with TEventLabelsKey.Create( Self ) do
  try
   RegDeleteKey( Handle, PChar( Self.Name ) );
  finally
   free;
  end;
end;

{ TStringProfile }

procedure TStringProfile.SetValue( NewValue :String );
var R :Longint;
begin
  if Key = nil then raise ERegistryError.CreateRes( SNoKey );
  R := RegSetValueEx( Key.Handle, PChar( Name ), 0, REG_SZ,
   PChar( NewValue ), Length( NewValue ) + 1 );
  if ( R <> ERROR_SUCCESS ) then raise ERegistryError.CreateRes( SRegWriteError );
end;

function TStringProfile.GetValue :String;
var S, R, T :Longint;
begin
  if Key = nil then raise ERegistryError.CreateRes( SNoKey );
  T := REG_SZ;
  R := RegQueryValueEx( Key.Handle, PChar( Name ), nil, @T, nil, @S );
  if ( R <> ERROR_SUCCESS ) then begin result := ''; exit; end;
  SetLength( result, S );
  try
    R := RegQueryValueEx( Key.Handle, PChar( Name ), nil, @T, pointer( result ), @S );
    if ( R <> ERROR_SUCCESS ) then raise ERegistryError.CreateRes( SRegReadError );
  except
    result := '';
    raise;
  end;
end;


{ TComponentProfile }

procedure TComponentProfile.SetValue( NewValue :TComponent );
var
  R :Longint;
  Stream :TMemoryStream;
begin
  if Key = nil then raise ERegistryError.CreateRes( SNoKey );
  if NewValue = nil then
   begin
     R := RegSetValueEx( Key.Handle, PChar( Name ), 0, REG_BINARY, nil, 0 );
     if ( R <> ERROR_SUCCESS ) then raise ERegistryError.CreateRes( SRegWriteError );
     exit;
   end;
  Stream := TMemoryStream.Create;
  try
    Stream.WriteComponent( NewValue );
    R := RegSetValueEx( Key.Handle, PChar( Name ), 0,
     REG_BINARY, Stream.Memory, Stream.Size );
    if ( R <> ERROR_SUCCESS ) then raise ERegistryError.CreateRes( SRegWriteError );
  finally
    Stream.Free;
  end;
end;

function TComponentProfile.GetValue :TComponent;
var
  S, R, T :Longint;
  Stream :TMemoryStream;
begin
  result := nil;
  if Key = nil then raise ERegistryError.CreateRes( SNoKey );
  T := REG_BINARY;
  R := RegQueryValueEx( Key.Handle, PChar( Name ), nil, @T, nil, @S );
  if ( R <> ERROR_SUCCESS ) or ( S = 0 ) then exit;
  Stream := TMemoryStream.Create;
  try
    Stream.SetSize( S + 32 );
    R := RegQueryValueEx( Key.Handle, PChar( Name ), nil, @T, Stream.Memory, @S );
    if ( R <> ERROR_SUCCESS ) then raise ERegistryError.CreateRes( SRegReadError );
    result := Stream.ReadComponent( nil );
  finally
    Stream.Free;
  end;
end;


{ TFormRestorer }

function TFormRestorer.GetPath :string;
begin
 if ParentKey = nil then result :=inherited GetPath
  else result := FindOwnerForm( Self ).Name;
end;

constructor TFormRestorer.Create( anOwner :TComponent );
var i :integer;
begin
 inherited Create( anOwner );
 FRestoring := frSizeAndPosition;
 FOldOnClose := nil;
 if ( csDesigning in ComponentState ) then
  for i := 0 to Screen.FormCount - 1 do
   begin
    FParentKey := TCustomKey( FindCompByClass( Screen.Forms[ i ], TAppConfigKey ) );
    if ( FParentKey <> Nil ) then break;
   end;
end;

procedure TFormRestorer.Loaded;
begin
 inherited Loaded;
 if ( not ( csDesigning in componentState ) ) then
  begin
    with FindOwnerForm( self ) do
     begin
       FOldOnClose := OnClose;
       OnClose := OurOnClose;
     end;
    if exist then restore;
  end;
end;

procedure TFormRestorer.Store;
var F :TForm; R :Longint; P :TRect;
begin
  F := FindOwnerForm( self );
  P := Rect( F.Left, F.Top, F.Width, F.Height );
  R := RegSetValueEx( Handle, Restorer_ValueName, 0, REG_BINARY, @P, SizeOf( P ) );
  if ( R <> ERROR_SUCCESS ) then raise ERegistryError.CreateRes( SRegWriteError );
end;

procedure TFormRestorer.Restore;
var S, R, T :Longint; P :TRect;
begin
  T := REG_BINARY;
  S := SizeOf( P );
  R := RegQueryValueEx( Handle, Restorer_ValueName, nil, @T, @P, @S );
  if ( R <> ERROR_SUCCESS ) or ( S <> SizeOf( P ) ) then exit;
  with FindOwnerForm( self ) do
   begin
    if Restoring in [ frSizeAndPosition, frPositionOnly ] then
      begin
        Left := P.Left;
        TOp := P.Top;
      end;
    if Restoring in [ frSizeAndPosition, frSizeOnly ] then
      begin
       Width := P.Right;
       Height := P.Bottom;
      end;
   end;
end;

procedure TFormRestorer.OurOnClose( Sender :TObject; var Action :TCloseAction );
begin
 Store;
 if assigned( FOldOnClose ) then FOldOnClose( Sender, Action );
end;

end.


